<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 * Author: Abdo Abouzaid <abouzaid@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 */

namespace Pinpoint\ProductEnquire\Plugin\Contact;

use Magento\Contact\Controller\Index\Post;
use Magento\Customer\Api\AccountManagementInterface as CustomerAccountManagement;
use Magento\Customer\Model\Session;
use Magento\Customer\Model\Url as CustomerUrl;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\Result\RedirectFactory;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\Validator\EmailAddress as EmailValidator;
use Magento\Newsletter\Controller\Subscriber\NewAction;
use Magento\Newsletter\Model\Subscriber;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Newsletter\Model\SubscriptionManagerInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\ProductEnquire\Helper\CustomEmailNotification;
use Pinpoint\ProductEnquire\Model\ProductEnquireFactory;
use Psr\Log\LoggerInterface;

class PostPlugin extends NewAction
{
    private const STORE_SENDER_NAME_CONFIG_PATH = 'trans_email/ident_general/name';
    private const STORE_SENDER_EMAIL_CONFIG_PATH = 'trans_email/ident_general/email';

    /**
     * @var LoggerInterface
     */
    private LoggerInterface $logger;

    /**
     * @var JsonFactory
     */
    protected JsonFactory $jsonFactory;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $messageManager;

    /**
     * @var ResultFactory
     */
    protected $resultRedirectFactory;

    /**
     * @var ScopeConfigInterface
     */
    protected ScopeConfigInterface $scopeConfig;

    /**
     * @var CustomEmailNotification
     */
    protected CustomEmailNotification $customerNotificationHelper;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected ResourceConnection $resource;
    /**
     * @var \Pinpoint\ProductEnquire\Model\ProductEnquireFactory
     */
    private ProductEnquireFactory $enquireFactory;
    /**
     * @var \Magento\Framework\Controller\Result\RedirectFactory
     */
    private RedirectFactory $redirectFactory;

    /**
     * Constructor
     * @param ProductEnquireFactory $enquireFactory
     * @param \Magento\Framework\Controller\Result\JsonFactory $jsonFactory
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param ResultFactory $resultRedirectFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param CustomEmailNotification $customerNotificationHelper
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param Context $context
     * @param SubscriberFactory $subscriberFactory
     * @param Session $customerSession
     * @param StoreManagerInterface $storeManager
     * @param CustomerUrl $customerUrl
     * @param CustomerAccountManagement $customerAccountManagement
     * @param EmailValidator $emailValidator
     * @param LoggerInterface $logger
     * @param \Magento\Framework\Controller\Result\RedirectFactory $redirectFactory
     * @param \Magento\Newsletter\Model\SubscriptionManagerInterface $subscriptionManager
     */
    public function __construct(
        ProductEnquireFactory        $enquireFactory,
        JsonFactory                  $jsonFactory,
        ManagerInterface             $messageManager,
        ResultFactory                $resultRedirectFactory,
        ScopeConfigInterface         $scopeConfig,
        CustomEmailNotification      $customerNotificationHelper,
        ResourceConnection           $resource,
        Context                      $context,
        SubscriberFactory            $subscriberFactory,
        Session                      $customerSession,
        StoreManagerInterface        $storeManager,
        CustomerUrl                  $customerUrl,
        CustomerAccountManagement    $customerAccountManagement,
        EmailValidator               $emailValidator,
        LoggerInterface              $logger,
        RedirectFactory              $redirectFactory,
        SubscriptionManagerInterface $subscriptionManager
    ) {
        $this->enquireFactory = $enquireFactory;
        $this->jsonFactory = $jsonFactory;
        $this->messageManager = $messageManager;
        $this->resultRedirectFactory = $resultRedirectFactory;
        $this->scopeConfig = $scopeConfig;
        $this->customerNotificationHelper = $customerNotificationHelper;
        $this->resource = $resource;
        $this->logger = $logger;
        $this->redirectFactory = $redirectFactory;

        parent::__construct(
            $context,
            $subscriberFactory,
            $customerSession,
            $storeManager,
            $customerUrl,
            $customerAccountManagement,
            $subscriptionManager,
            $emailValidator
        );
    }

    /**
     * Post user question
     * @return \Magento\Framework\Controller\Result\Redirect|\Magento\Framework\Controller\Result\Json
     */
    public function afterExecute(Post $subject)
    {
        if ($subject->getRequest()->isXmlHttpRequest()) {
            $response = $this->jsonFactory->create();
            $errorMessages = $this->messageManager->getMessages()->getErrors();
            if (!empty($errorMessages)) {
                return $response->setData([
                    'Message' => $errorMessages[0]->getText()
                ]);
            }
            $connection = $this->resource->getConnection();
            try {
                $connection->beginTransaction();
                $data = $subject->getRequest()->getParams();
                $enquiry = $this->enquireFactory->create();
                $enquiry->setData($data);
                $enquiry->save();
                $this->sendCustomerNotification($data);
                $this->messageManager->getMessages(true);
                $this->messageManager->addSuccessMessage(
                    __('Your inquiry was submitted and will be responded to as soon as possible')
                );
                $response->setData(['Message' => 'Request Submitted']);

                if (isset($data['subscribe'])) {
                    $this->subscribe($data['email']);
                }
                $connection->commit();
            } catch (\Exception $e) {
                $this->logger->critical($e->getMessage());
                $connection->rollBack();
            }

            return $response;
        }

        return $this->redirectFactory->create()->setPath($this->_redirect->getRefererUrl());
    }

    /**
     * Send thank you email to customer
     * @param array $data
     * @return void
     */
    protected function sendCustomerNotification(array $data): void
    {
        /* Receiver Detail  */
        $receiverInfo = [
            'name' => $data['name'],
            'email' => $data['email']
        ];

        /* Sender Detail  */
        $senderInfo = [
            'name' => $this->scopeConfig->getValue(self::STORE_SENDER_NAME_CONFIG_PATH, ScopeInterface::SCOPE_STORE),
            'email' => $this->scopeConfig->getValue(self::STORE_SENDER_EMAIL_CONFIG_PATH, ScopeInterface::SCOPE_STORE),
        ];

        $emailTemplateVariables = ['data' => new DataObject($data)];

        try {
            $this->customerNotificationHelper->notifyCustomer(
                $emailTemplateVariables,
                $senderInfo,
                $receiverInfo
            );
        } catch (NoSuchEntityException|LocalizedException $e) {
        }
    }

    /**
     * Email Subscription
     * @param string $email
     * @return void
     * @throws \Exception
     */
    protected function subscribe(string $email): void
    {
        $this->validateEmailFormat($email);
        $this->validateGuestSubscription();
        $this->validateEmailAvailable($email);

        $subscriber = $this->_subscriberFactory->create()->loadByEmail($email);
        if (!($subscriber->getId() && $subscriber->getSubscriberStatus() === Subscriber::STATUS_SUBSCRIBED)) {
            $this->_subscriberFactory->create()->subscribe($email);
        }
    }
}
